# Generated by CodiumAI
# Dependencies:
# pip install pytest-mock
import pytest

from pyadtpulse.alarm_panel import ADTPulseAlarmPanel
from pyadtpulse.const import DEFAULT_API_HOST
from pyadtpulse.gateway import ADTPulseGateway
from pyadtpulse.pulse_connection import ADTPulseConnection
from pyadtpulse.site import ADTPulseSite
from pyadtpulse.zones import ADTPulseZones


class TestADTPulseSite:
    # can create an instance of ADTPulseSite with pulse_connection, site_id, and name
    def test_create_instance(self):
        pulse_connection = ADTPulseConnection(DEFAULT_API_HOST)
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        assert site._pulse_connection == pulse_connection
        assert site._id == "site_id"
        assert site._name == "site_name"
        assert site._last_updated == 0
        assert isinstance(site._zones, ADTPulseZones)
        # assert isinstance(site._site_lock, RLock) or isinstance(site._site_lock, DebugRLock)
        assert isinstance(site._alarm_panel, ADTPulseAlarmPanel)
        assert isinstance(site._gateway, ADTPulseGateway)

    # can get the id of the site
    def test_get_id(self):
        pulse_connection = ADTPulseConnection(DEFAULT_API_HOST)
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        assert site.id == "site_id"

    # can get the name of the site
    def test_get_name(self):
        pulse_connection = ADTPulseConnection(DEFAULT_API_HOST)
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        assert site.name == "site_name"

    # can get the last_updated timestamp of the site
    def test_get_last_updated(self):
        pulse_connection = ADTPulseConnection(DEFAULT_API_HOST)
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        assert site.last_updated == 0

    # can get the site_lock of the site
    """
    @pytest.mark.asyncio
    def test_get_site_lock(self):
        pulse_connection = ADTPulseConnection(DEFAULT_API_HOST)
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        assert isinstance(site.site_lock, RLock) or isinstance(site.site_lock, DebugRLock)
        """

    # can arm the system home
    def test_arm_home(self, mocker):
        pulse_connection = ADTPulseConnection(DEFAULT_API_HOST)
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        site.arm_home() == True

    # Raises a RuntimeError if no control panels exist when trying to arm the system home
    def test_arm_home_no_control_panels(self, mocker):
        pulse_connection = ADTPulseConnection(DEFAULT_API_HOST)
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        with pytest.raises(RuntimeError):
            site.arm_home()

    # Raises a RuntimeError if no zones exist when trying to get the zones of the site as a list of ADTPulseFlattendZone objects
    def test_get_zones_list_no_zones(self, mocker):
        pulse_connection = ADTPulseConnection(DEFAULT_API_HOST)
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        with pytest.raises(RuntimeError):
            site.zones

    # Raises a RuntimeError if no zones exist when trying to get the zones of the site as a dictionary of ADTPulseZone objects
    def test_get_zones_dict_no_zones(self, mocker):
        pulse_connection = ADTPulseConnection(DEFAULT_API_HOST)
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        with pytest.raises(RuntimeError):
            site.zones_as_dict

    # can arm the system away
    @pytest.mark.asyncio
    def test_arm_away(self, event_loop):
        pulse_connection = ADTPulseConnection(DEFAULT_API_HOST)
        pulse_connection.loop = event_loop
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        assert site.arm_away() is True

    # can disarm the system
    def test_disarm_system_fixed(self, mocker):
        pulse_connection = mocker.Mock(spec=ADTPulseConnection)
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")

        with pytest.raises(RuntimeError):
            site.disarm()

        site._alarm_panel = mocker.Mock(spec=ADTPulseAlarmPanel)
        site._alarm_panel.disarm.return_value = True

        assert site.disarm() == True

    # can asynchronously arm the system home
    @pytest.mark.asyncio
    async def test_async_arm_home(self, mocker):
        pulse_connection = mocker.Mock(spec=ADTPulseConnection)
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")

        site._alarm_panel = mocker.Mock(spec=ADTPulseAlarmPanel)
        site._alarm_panel.async_arm_home.return_value = True

        assert await site.async_arm_home() == True

    # can asynchronously arm the system away
    @pytest.mark.asyncio
    async def test_async_arm_away(self, mocker):
        pulse_connection = mocker.Mock(spec=ADTPulseConnection)
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        site._alarm_panel = mocker.Mock(spec=ADTPulseAlarmPanel)
        site._alarm_panel.async_arm_away.return_value = True

        assert await site.async_arm_away() is True

    # can asynchronously disarm the system
    @pytest.mark.asyncio
    async def test_disarm_system(self, mocker):
        # Arrange
        pulse_connection = mocker.Mock()
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        alarm_control_panel = mocker.Mock()
        site.alarm_control_panel = alarm_control_panel

        # Act
        result = await site.async_disarm()

        # Assert
        alarm_control_panel.disarm.assert_called_once_with(pulse_connection)
        assert result == alarm_control_panel.disarm.return_value

    # can get the zones of the site as a list of ADTPulseFlattendZone objects
    def test_get_zones_as_list(self, mocker):
        # Arrange
        pulse_connection = mocker.Mock()
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        zones = mocker.Mock()
        site._zones = zones
        flattened_zones = mocker.Mock()
        zones.flatten.return_value = flattened_zones

        # Act
        result = site.zones

        # Assert
        zones.flatten.assert_called_once_with()
        assert result == flattened_zones

    # can get the zones of the site as a dictionary of ADTPulseZone objects
    def test_get_zones_as_dict(self, mocker):
        # Arrange
        pulse_connection = mocker.Mock()
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        zones = mocker.Mock()
        site._zones = zones

        # Act
        result = site.zones_as_dict

        # Assert
        assert result == zones

    # can get the alarm control panel of the site
    def test_get_alarm_control_panel(self):
        pulse_connection = ADTPulseConnection(DEFAULT_API_HOST)
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        assert isinstance(site.alarm_control_panel, ADTPulseAlarmPanel)

    # can get the gateway of the site
    def test_get_gateway(self):
        pulse_connection = ADTPulseConnection(DEFAULT_API_HOST)
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        assert isinstance(site.gateway, ADTPulseGateway)

    # raises a RuntimeError if no control panels exist when trying to arm the system home
    def test_raise_runtime_error_when_no_control_panels_exist(self):
        pulse_connection = ADTPulseConnection(DEFAULT_API_HOST)
        site = ADTPulseSite(pulse_connection, "site_id", "site_name")
        with pytest.raises(RuntimeError):
            site.arm_home()



"""ADT Pulse Gateway Dataclass."""

import logging
from dataclasses import dataclass
from ipaddress import IPv4Address, IPv6Address, ip_address
from threading import RLock
from typing import Any

from .const import ADT_DEFAULT_POLL_INTERVAL, ADT_GATEWAY_OFFLINE_POLL_INTERVAL
from .util import parse_pulse_datetime

LOG = logging.getLogger(__name__)

STRING_UPDATEABLE_FIELDS = (
    "manufacturer",
    "model",
    "serial_number",
    "firmware_version",
    "hardware_version",
    "primary_connection_type",
    "broadband_connection_status",
    "cellular_connection_status",
    "cellular_connection_signal_strength",
    "broadband_lan_mac",
    "device_lan_mac",
)

DATETIME_UPDATEABLE_FIELDS = ("next_update", "last_update")

IPADDR_UPDATEABLE_FIELDS = (
    "broadband_lan_ip_address",
    "device_lan_ip_address",
    "router_lan_ip_address",
    "router_wan_ip_address",
)


@dataclass(slots=True)
class ADTPulseGateway:
    """ADT Pulse Gateway information."""

    manufacturer: str = "Unknown"
    _status_text: str = "OFFLINE"
    _current_poll_interval: float = ADT_DEFAULT_POLL_INTERVAL
    _initial_poll_interval: float = ADT_DEFAULT_POLL_INTERVAL
    _attribute_lock = RLock()
    model: str | None = None
    serial_number: str | None = None
    next_update: int = 0
    last_update: int = 0
    firmware_version: str | None = None
    hardware_version: str | None = None
    primary_connection_type: str | None = None
    broadband_connection_status: str | None = None
    cellular_connection_status: str | None = None
    cellular_connection_signal_strength: float = 0.0
    broadband_lan_ip_address: IPv4Address | IPv6Address | None = None
    broadband_lan_mac: str | None = None
    device_lan_ip_address: IPv4Address | IPv6Address | None = None
    device_lan_mac: str | None = None
    router_lan_ip_address: IPv4Address | IPv6Address | None = None
    router_wan_ip_address: IPv4Address | IPv6Address | None = None

    @property
    def is_online(self) -> bool:
        """Returns whether gateway is online.

        Returns:
            bool: True if gateway is online
        """
        with self._attribute_lock:
            return self._status_text == "ONLINE"

    @is_online.setter
    def is_online(self, status: bool) -> None:
        """Set gateway status.

        Args:
            status (bool): True if gateway is online

        Also changes the polling intervals
        """
        with self._attribute_lock:
            if status == self.is_online:
                return

            self._status_text = "ONLINE"
            if not status:
                self._status_text = "OFFLINE"
                self._current_poll_interval = ADT_GATEWAY_OFFLINE_POLL_INTERVAL
            else:
                self._current_poll_interval = self._initial_poll_interval

            LOG.info(
                "ADT Pulse gateway %s, poll interval=%f",
                self._status_text,
                self._current_poll_interval,
            )

    @property
    def poll_interval(self) -> float:
        """Set polling interval.

        Returns:
            float: number of seconds between polls
        """
        with self._attribute_lock:
            return self._current_poll_interval

    @poll_interval.setter
    def poll_interval(self, new_interval: float | None) -> None:
        """Set polling interval.

        Args:
            new_interval (float): polling interval if gateway is online,
                if set to None, resets to ADT_DEFAULT_POLL_INTERVAL

        Raises:
            ValueError: if new_interval is less than 0
        """
        if new_interval is None:
            new_interval = ADT_DEFAULT_POLL_INTERVAL
        elif new_interval < 0.0:
            raise ValueError("ADT Pulse polling interval must be greater than 0")
        with self._attribute_lock:
            self._initial_poll_interval = new_interval
            if self._current_poll_interval != ADT_GATEWAY_OFFLINE_POLL_INTERVAL:
                self._current_poll_interval = new_interval
            LOG.debug("Set poll interval to %f", self._initial_poll_interval)

    def adjust_backoff_poll_interval(self) -> None:
        """Calculates the backoff poll interval.

        Each call will adjust current_poll interval with exponential backoff,
        unless gateway is online, in which case, poll interval will be reset to
        initial_poll interval."""

        with self._attribute_lock:
            if self.is_online:
                self._current_poll_interval = self._initial_poll_interval
                return
            # use an exponential backoff
            self._current_poll_interval = self._current_poll_interval * 2
            if self._current_poll_interval > ADT_GATEWAY_OFFLINE_POLL_INTERVAL:
                self._current_poll_interval = ADT_DEFAULT_POLL_INTERVAL
            LOG.debug(
                "Setting current poll interval to %f", self._current_poll_interval
            )

    def set_gateway_attributes(self, gateway_attributes: dict[str, str]) -> None:
        """Set gateway attributes from dictionary.

        Args:
            gateway_attributes (dict[str,str]): dictionary of gateway attributes
        """ """"""
        for i in (
            STRING_UPDATEABLE_FIELDS
            + IPADDR_UPDATEABLE_FIELDS
            + DATETIME_UPDATEABLE_FIELDS
        ):
            temp: Any = gateway_attributes.get(i)
            if temp == "":
                temp = None
            if temp is None:
                setattr(self, i, None)
                continue
            if i in IPADDR_UPDATEABLE_FIELDS:
                try:
                    temp = ip_address(temp)
                except ValueError:
                    temp = None
            elif i in DATETIME_UPDATEABLE_FIELDS:
                try:
                    temp = int(parse_pulse_datetime(temp).timestamp())
                except ValueError:
                    temp = None
            setattr(self, i, temp)
